#include "cb-tsu.h"
//#include <stdio.h>

/******************************************************************************
 *
 *  C - Startupcode for H8/300H
 *
 ******************************************************************************
 */

asm ("  	.h8300h                                                     ");
asm ("  	.section .text                                              ");
asm ("  	.global	_start                                              ");
asm ("  _start:                                                         ");
asm ("  	mov.l	#_stack,sp			; set stackpointer              ");
asm ("  	jsr		_low_level_init                                     ");
asm ("                                                                  ");
asm ("  								; copy rom image to ram for     ");
asm ("  								; initialised data segments     ");
asm ("  	mov.l   #_mdata,er0				; start of rom image        ");
asm ("  	mov.l   #_data,er1				; start of data segment     ");
asm ("  	mov.l   #_edata,er2				; end of data segment       ");
asm ("  .loop2:                                                         ");
asm ("  	mov.w	@er0,r3                                             ");
asm ("  	mov.w	r3,@er1                                             ");
asm ("  	adds	#2,er0                                              ");
asm ("  	adds	#2,er1                                              ");
asm ("  	cmp.l	er2,er1                                             ");
asm ("  	blo	.loop2                                                  ");
asm ("                                                                  ");
asm ("  								; blank zero init data segement ");
//asm ("  	mov.l	#_edata,er0			                                ");
//asm ("  	mov.l	#_end,er1                                           ");
//asm ("  	mov.w	#0,r2		; not sure about alignment requirements ");
//asm ("  .loop:	mov.w	r2,@er0		; playing it safe for now           ");
//asm ("  	adds	#2,er0                                              ");
//asm ("  	cmp.l	er1,er0                                             ");
//asm ("  	blo	.loop                                                   ");
asm ("  	mov.l	#_bss,er0			                                ");
asm ("  	mov.l	#_ebss,er1                                           ");
asm ("  	mov.w	#0,r2		; not sure about alignment requirements ");
asm ("  .loop:	mov.w	r2,@er0		; playing it safe for now           ");
asm ("  	adds	#2,er0                                              ");
asm ("  	cmp.l	er1,er0                                             ");
asm ("  	blo	.loop                                                   ");
asm ("  	jsr	@___main                                                ");
asm ("  	jsr	@_main                                                  ");
asm ("  	jsr	@_exit                                                  ");
asm ("                                                                  ");
asm ("  	.section .stack                                             ");
asm ("  _stack:	.long 	1                                               ");
asm ("                                                                  ");
asm ("      .section .text                                              ");




/************************************************************************************************************
*                                                                                                           *
*   Init Shadow - Vector- Table:                                                                            *
*   ----------------------------                                                                            *
*   Update the interrupt vector table to JMP @aa:24                                                         *
*   The interrupt vector table contains the address "h'00xxxxxx"                                            *
*   JMP @aa:24, Setting JMP instruction "h'5axxxxxx"                                                        *
*                                                                                                           *
************************************************************************************************************/

void Init_Vector_Table(void)
{
 int c;

  for (c=0; c<64; c++)
   {
   (*(volatile unsigned char *)(0x20000L+(unsigned long)(c*4))) = (unsigned char)0x5a;
   }
}



/************************************************************************************************************
*                                                                                                           *
*    Grund- Intitialisierungen fr den Hitachi H8/300 - 3067F                                               *
*    --------------------------------------------------------                                               *
*                                                                                                           *
*    DATUM / AUTOR:     19.08.2000 - Philippe Gantenbein                                                    *
*    FILENAME:          LOWINIT.C                                                                           *
*                                                                                                           *
*    ENT.- UMGEBUNG:    HITACHI / WORKBENCH Demo Version                                                    *
*                                                                                                           *
*    Target - System:   EVB H8/300 3067F                                                                    *
*                                                                                                           *
*    COPYRIGHT:         Philippe Gantenbein / Rene Gtti Cooperation                                        *
*                       Computer & Electronics                                                              *
*                                                                                                           *
*************************************************************************************************************
*                                                                                                           *
*    BESCHREIBUNG:                                                                                          *
*    -------------                                                                                          *
*    LOW Level Initialisierungen fr den Controller. In diesem Unterprogramm mssen alle Portdefinitionen   *
*    gemacht werden. Das heisst, dass der Datenbus, Adressbus und der Controllbus initialisiert werden muss.*
*    Dass der Externe Waitcontroller richtig funktioniert muss auf dessen Adressbereich minimum 1 Wait-     *
*    State eingestellt werden.                                                                              *
*                                                                                                           *
*************************************************************************************************************
*                                                                                                           *
*    Aenderungen:    Version:   Wer:           Wann:           Was:                                         *
*    ------------                                                                                           *
*                    1.00a      P.Gantenbein   19.08.2000      Erste freigegebene Version                   *
*                                                                                                           *
************************************************************************************************************/

//#pragma language=extended

#include "ioh83067.h"                            //* Register Adressen HITACHI H8/3067F


void low_level_init(void)
{

/************************************************************************************************************
*                                                                                                           *
*    Initialisierung der Ports                                                                              *
*                                                                                                           *
************************************************************************************************************/ 

  P1DDR  = 0xFF;                                 //* P1 => Adress-Leitungen A[ 0.. 7] 
  P2DDR  = 0xFF;                                 //* P2 => Adress-Leitungen A[ 8..15]
  P5DDR  = 0xFF;                                 //* P5 => Adress-Leitungen A[16..19]
  
//*P4DDR = 0xFF;                                 //* P4 => Daten-Leitungen  D[ 0.. 7] ->automatisch
//*P3DDR = 0xFF;                                 //* P3 => Daten-Leitungen  D[ 8..15] ->automatisch
  
  P6DDR  = 0x06;                                 //* Testpunkt 1 und FWE Ansteuerung
  P8DDR  = 0x14;                                 //* CS0, CS2, IRQ0, IRQ1, ADTRG

  PADDR  = 0x00;                                 //* ganzes Port als Input
  PBDDR  = 0x26;                                 //* LCD Back, BAT OK, TP2, SDA, SCL

/************************************************************************************************************
*                                                                                                           *
*    Initialisierung der Bussteuerung                                                                       *
*                                                                                                           *
************************************************************************************************************/ 

  ABWCR &= ~0x01;                                //* Onboard RAM, Bus Breite 16 Bit
  ABWCR |=  0x14;                                //* Onboard EPLD und Externer Bus, Bus Breite 8 Bit
  
  ASTCR |=  0x15;                                //* Onboard RAM, EPLD und Externer Bus 3 State Zugriff
  WCRH  |=  0x03;                                //* Externer Bus Area 4 mit 3 Wait-States
  WCRL  |=  0x30;                                //* Onboard EPLD Area 2 mit 3 Wait-States
  WCRL  &= ~0x03;                                //* Onboard RAM  Area 0 mit 0 Wait-States

  BRCR   =  0xF0;                                //* Keine A20..A23 Adressleitungen, kein Busrelase
  BCR   |=  0x01;                                //* Wait Pin Enable
  CSCR  |=  0x1F;                                //* Enable Chipselekt 4, Externer Bus

  Init_Vector_Table();	
}


/************************************************************************************************************
*                                                                                                           *
*    Initialisierung der seriellen Schnitstelle 1 fr getchar() und putchar()                               *
*                                                                                                           *
************************************************************************************************************/ 
void CBTSU_RSInit(void)
{
  SCI_SMR1  = 0x00;                              //* Mode: Asynch, 8 Data, None, 1 Stop, Phi
  SCI_BRR1  = 0x3B;                              //* Baudrate 9600Bd
  SCI_SCR1  = 0x30;                              //* CTR:  Rx/Tx enabled, IRQ disabled, CLK am SCK-Pin
  SCI_SSR1 &= 0x84;                              //* STS:  Reset Status-Flags

}




int getchar (void)                                   //* empfange ein Zeichen
 {
 unsigned char ch1;                                            //* Variable fr empfangenes Zeichen
 
 while ((SCI_SSR1 & 0x40) != 0x40)                             //* Warte bis Zeichen Empfangen
  {
  if ((SCI_SSR1 & 0x38) != 0x00)                               //* ist ein Empfangsfehler aufgetreten ??
   SCI_SSR1 &= 0xC7;                                           //* lsche alle Empfangsfehler Flag's
  }
 ch1 = SCI_RDR1;                                               //* empfangenes Zeichen speichern
 SCI_SSR1 &= 0xBF;                                             //* lsche  empfangs Register voll Flag
 return ((int)ch1);                                                 //* Funktionsrckgabe -> empfangenes Zeichen
}


int putchar (int  c)                                 //* sende ein Zeichen
 {
 while ((SCI_SSR1 & 0x80) != 0x80);                            //* warte sende Daten Register leer Flag = 1
 SCI_TDR1 = (unsigned char)c;                                                 //* Zeichen in Transmit Register
 SCI_SSR1 &= 0x7F;                                             //* sende Daten Register leer Flag lschen
 return(c);
 }



void CBTSU_RSWriteLn(char *s)
{
while (*s)
	{
	if (*s == 0x0a)
	   putchar(0x0d);	
	putchar(*s++);
	}
}

void CBTSU_RSReadLn(char *s)
{
char c;
do {
	c = getchar();		// Zeichen holen
	if (c != 0x0a)		// Newline verwerfen
		*s++ = c;
   }
while (c != 0x0d);
s--;
*s = '\0';
}


/************************************************************************

  Interrupt vectors section - SH1

  created 6-Sep-95 :: PBB
************************************************************************/

/***********************************************************************
  Place the prototypes for interrupt service routines here
*/

void start (void); /* Startup code (in start.s)  */
void stack (void); /* infact this is a constant! */
void isr   (void);
void ISR_IRQ0(void);
/***********************************************************************
  Typedef for the function pointer
*/

typedef void (*fp) (void);

/***********************************************************************
  Place the interrupt service routine symbols in the table 
  to create the vector entry
*/

const fp HardwareVectors[] __attribute__ ((section (".vects"))) = {
  start,        /* 0  Reset vector */
  (fp)0L,       /* 1  Reserved     */
  (fp)0L,       /* 2  Reserved     */
  (fp)0L,       /* 3  Reserved     */
  (fp)0L,       /* 4  Reserved     */
  (fp)0L,       /* 5  Reserved     */
  (fp)0L,       /* 6  Reserved     */
  (fp)0L,       /* 7  NMI          */
  (fp)0L,       /* 8  Trap 0       */
  (fp)0L,       /* 9  Trap 1       */
  (fp)0L,       /* 10 Trap 2       */
  (fp)0L,       /* 11 Trap 3       */
  (fp)0L,       /* 12 Ext. Int IRQ0*/
  (fp)0L,       /* 13 Ext. Int IRQ1*/
  (fp)0L,       /* 14 Ext. Int IRQ2*/
  (fp)0L,       /* 15 Ext. Int IRQ3*/
  (fp)0L,       /* 16 Ext. Int IRQ4*/
  (fp)0L,       /* 17 Ext. Int IRQ5*/
  (fp)0L,       /* 18 Reserved     */
  (fp)0L,       /* 19 Reserved     */
  (fp)0L,       /* 20 WOVI   Watchdog           */
  (fp)0L,       /* 21 CMI    DRAM interface     */
  (fp)0L,       /* 22 Reserved                  */
  (fp)0L,       /* 23 ADI    A/D end            */
  (fp)0L,       /* 24 IMIA0  16bit timer0(compA)*/
  (fp)0L,       /* 25 IMIB0  16bit timer0(compB)*/
  (fp)0L,       /* 26 OVI0   16bit timer0(overf)*/
  (fp)0L,       /* 27 Reserved                  */
  (fp)0L,       /* 28 IMIA1  16bit timer1(compA)*/
  (fp)0L,       /* 29 IMIB1  16bit timer1(compB)*/
  (fp)0L,       /* 30 OVI1   16bit timer1(overf)*/
  (fp)0L,       /* 31 Reserved                  */
  (fp)0L,       /* 32 IMIA2  16bit timer2(compA)*/
  (fp)0L,       /* 33 IMIB2  16bit timer2(compB)*/
  (fp)0L,       /* 34 OVI2   16bit timer2(overf)*/
  (fp)0L,       /* 35 Reserved                  */
  (fp)0L,       /* 36 CMIA0  8bit timer01(compA)*/
  (fp)0L,       /* 37 CMIB0  8bit timer01(compB)*/
  (fp)0L,       /* 38 CMIA1/CMIB1  8bit timer01(compAB)*/
  (fp)0L,       /* 39 TOVI0/TOVI1  8bit timer01(overf) */
  (fp)0L,       /* 40 CMIA2  8bit timer23(compA)*/
  (fp)0L,       /* 41 CMIB2  8bit timer23(compB)*/
  (fp)0L,       /* 42 CMIA3/CMIB3  8bit timer23(compAB)*/
  (fp)0L,       /* 43 TOVI2/TOVI3  8bit timer23(overf) */
  (fp)0L,       /* 44 DEND0A DMA controller     */
  (fp)0L,       /* 45 DEND0B DMA controller     */
  (fp)0L,       /* 46 DEND1A DMA controller     */
  (fp)0L,       /* 47 DEND1B DMA controller     */
  (fp)0L,       /* 48 Reserved                  */
  (fp)0L,       /* 49 Reserved                  */
  (fp)0L,       /* 50 Reserved                  */
  (fp)0L,       /* 51 Reserved                  */
  (fp)0L,       /* 52 ERI0   SCI 0 (rcv error)  */
  (fp)0L,       /* 53 RXI0   SCI 0 (rcv full)   */
  (fp)0L,       /* 54 TXI0   SCI 0 (trn empty)  */
  (fp)0L,       /* 55 TEI0   SCI 0 (trn error)  */
  (fp)0L,       /* 56 ERI1   SCI 1 (rcv error)  */
  (fp)0L,       /* 57 RXI1   SCI 1 (rcv full)   */
  (fp)0L,       /* 58 TXI1   SCI 1 (trn empty)  */
  (fp)0L,       /* 59 TEI1   SCI 1 (trn error)  */
  (fp)0L,       /* 60 ERI2   SCI 2 (rcv error)  */
  (fp)0L,       /* 61 RXI2   SCI 2 (rcv full)   */
  (fp)0L,       /* 62 TXI2   SCI 2 (trn empty)  */
  (fp)0L,       /* 63 TEI2   SCI 2 (trn error)  */
};

/* Done...
************************************************************************/


int __main() {}

int exit()
{
/* put your user exit code here */
while(1);
}

int end() 
{
while(1);
}


/************************************************************************************************************
*                                                                                                           *
*    Display- Routinen fr den Hitachi H8/300 - 3067F                                                       *
*    ------------------------------------------------                                                       *
*                                                                                                           *
*    DATUM / AUTOR:     19.08.2000 - Philippe Gantenbein                                                    *
*    FILENAME:          HD44780.C                                                                           *
*                                                                                                           *
*    ENT.- UMGEBUNG:    HITACHI / WORKBENCH Demo Version                                                    *
*                                                                                                           *
*    Target - System:   EVB H8/300 3067F                                                                    *
*                                                                                                           *
*    COPYRIGHT:         Philippe Gantenbein / Rene Gtti Cooperation                                        *
*                       Computer & Electronics                                                              *
*                                                                                                           *
*************************************************************************************************************
*                                                                                                           *
*    BESCHREIBUNG:                                                                                          *
*    -------------                                                                                          *
*                                                                                                           *
*    Routinen um ein LCD Display mit dem Controller HD44780 anzusprechen. Folgende Display- Typen werden    *
*    Untersttzt: -> LM015 ... LM044L                                                                       *
*    ---------------------------------------------------------------------------------------------------    *
*    Bin_Str:           Wandelt eine Binrzahl in einen String mit 3 Byte Lnge plus Endmarke '\0' um.      *
*                       Mit TRUE knnen fhrende Nullen anzeigen werden oder mit FALSE ausgeblendet werden. *
*                       Datentyp der Binrzahl -> unsigned char.                                            *
*                                                                                                           *
*    LCD_Init:          Initialisierung des LCD's mit dem angegebenem Typ.                                  *
*                                                                                                           *
*    LCD_Char:          Schreibt ein Zeichen bei aktueller Cursor Position auf das LCD.                     *
*                                                                                                           *
*    LCD_String:        Schreibt ein String mittels LCD_Char aus das LCD.                                   *
*                                                                                                           *
*    LCD_Write:         Schreibt kombiniert, String mit Platzhalter fr Zahl (1Byte) auf das LCD.           *
*                       Platzhalter -> %c                                                                   *
*                                                                                                           *
*    LCD_XY:            Positioniert den Cursor auf dem LCD.                                                *
*                                                                                                           *
*                                                                                                           *
*    Makros fr die Hintergrund - Beleuchtung                                                               *
*    ----------------------------------------                                                               *
*    LCD_Back_ON()      Einschalten der Hintergrundbeleuchtung                                              *
*                                                                                                           *
*    LCD_Back_OFF()     Ausschalten der Hintergrundbeleuchtung                                              *
*                                                                                                           *
*    LCD_Back_TOGGLE()  Toggeln der Hintergrundbeleuchtung                                                  *
*                                                                                                           *
*************************************************************************************************************
*                                                                                                           *
*    Aenderungen:    Version:   Wer:           Wann:           Was:                                         *
*    ------------                                                                                           *
*                    1.00a      P.Gantenbein   19.08.2000      Erste freigegebene Version                   *
*                                                                                                           *
************************************************************************************************************/

#pragma language=extended

#define LCD_Inst (*(volatile unsigned char*)(0x400000))        //* Adresse fr LCD Instruktionen
#define LCD_Data (*(volatile unsigned char*)(0x400001))        //* Adresse fr LCD Daten

enum boolean {FALSE,TRUE};                                     //* Boolsche Konstanten
static enum boolean InitDone = FALSE;


/************************************************************************************************************
*                                                                                                           *
*    LCD - Instruktionen schreiben                                                                          *
*                                                                                                           *
************************************************************************************************************/ 

static void LCD_IWrite(unsigned char instruction)
 {
 unsigned char Busy;                                           //* Dummy Byte fr Busy- Flag
 
 LCD_Inst = instruction;                                       //* Instruktion an LCD senden
 do
  {
  Busy=LCD_Inst;                                               //* LCD intruktionen lesen
  }
 while (Busy > 0x7F);                                          //* wiederhole bis Busy- Flag 
 }                                                             //* zurckgesetzt ist


/************************************************************************************************************
*                                                                                                           *
*    LCD - Initialisieren                                                                                   *
*                                                                                                           *
************************************************************************************************************/ 

static void Wait()                                                    //* Verzgerungsschlaufe
 {
 unsigned int i;                                               //* Hilfsvariable zum Zhlen
 unsigned int j;                                               //* Hilfsvariable zum Zhlen

 for (i=0;i<800;i++)                                           //* ussere Zhlerschlaufe
  for (j=0;j<800;j++);                                         //* innere  Zhlerschlaufe
 }


static void LCD_Init(void)
 {
 unsigned char type =0x38;

 Wait();                                                       //* Warte
 LCD_Inst = type;                                              //* LCD Type Init 1
 Wait();                                                       //* Warte
 LCD_Inst = type;                                              //* LCD Type Init 2
 Wait();                                                       //* Warte
 LCD_Inst = type;                                              //* LCD Type Init 3
 LCD_Inst = type;                                              //* LCD mit Type Konfigurieren
 LCD_IWrite(0x08);                                             //* Display ON/OFF Control ->OFF
 LCD_IWrite(0x01);                                             //* Clear Display
 LCD_IWrite(0x06);                                             //* Entry Mode ->Increment, No shift

//* LCD_IWrite(0x0F);                                             //* Display ON/OFF Control ->ON
//*                                                               //* Cursor -> ON     Blink ->ON

 LCD_IWrite(0x0C);                                             //* Display ON/OFF Control ->ON
                                                               //* Cursor -> OFF    Blink ->OFF
 InitDone = TRUE;
 }


/************************************************************************************************************
*                                                                                                           *
*    Schreibe Zeichen auf LCD                                                                               *
*                                                                                                           *
************************************************************************************************************/

static void LCD_Char(char c)
 {
 unsigned char Busy;                                           //* Dummy Byte fr Busy- Flag

 LCD_Data=c;                                                   //* Daten an LCD senden
 do
  {
  Busy=LCD_Inst;                                               //* LCD intruktionen lesen
  }
 while (Busy > 0x7F);                                          //* wiederhole bis Busy- Flag 
                                                               //* zurckgesetzt ist
 }



/************************************************************************************************************
*                                                                                                           *
*    Positioniere Cursor auf LCD                                                                            *
*                                                                                                           *
************************************************************************************************************/

static void LCD_XY(unsigned char x,unsigned char y)
 {
 switch (y)                                                    //* LCD- Zeile auswhlen
  {                                                            //**********************************
  case 1:       x=x+0x7F;
                LCD_IWrite(x);                                 //* LCD 1. Zeile 0x80h..0x93h
                break;
  case 2:       x=x+0xBF;
                LCD_IWrite(x);                                 //* LCD 2. Zeile 0xC0h..0xD3h
                break;
  case 3:       x=x+0x93;
                LCD_IWrite(x);                                 //* LCD 3. Zeile 0x94h..0xA7h
                break;
  case 4:       x=x+0xD3;
                LCD_IWrite(x);                                 //* LCD 4. Zeile 0xD4h..0xE7h
                break;
  }
 }


/************************************************************************************************************
*                                                                                                           *
*    Schreibe Zeichenkette auf LCD                                                                          *
*                                                                                                           *
************************************************************************************************************/

//void LCD_DispString(unsigned char zeile, char *s)
void CBTSU_DispString(unsigned char zeile, char *s)
 {
 if (!InitDone)	LCD_Init();
 LCD_XY(1,zeile);
 while (*s != '\0')                                            //* Prfe auf Stringende
  {
  LCD_Char(*s);                                                //* Zeichen auf LCD schreiben
  s++;                                                         //* Pointer inkrtementieren
  }
 }


/************************************************************************************************************
*                                                                                                           *
*    Schreibe Integer auf LCD                                                                          *
*                                                                                                           *
************************************************************************************************************/

//void LCD_DispInt(unsigned char zeile, int n)
void CBTSU_DispInt(unsigned char zeile, int n)
 {
 char str[8];
 char* s = str;
 char i;
 char v=0;

 if (!InitDone)	LCD_Init();
 //itoa(n,str);
 //sprintf(str,"%6d",n);
 for (i=0;i<8;i++)
	*s++ = ' ';
 --s;
 *s = '\0';
 --s;
 *s = '0';
 if (n<0)  {n=-n; v =1;}
 while (n>0)
	{
	*s-- = '0' + (n%10);
	n /= 10;
	}
 if	(v)	*s = '-';

 LCD_XY(1,zeile);
 s = str;
 while (*s != '\0')                                            //* Prfe auf Stringende
  {
  LCD_Char(*s);                                                //* Zeichen auf LCD schreiben
  s++;                                                         //* Pointer inkrtementieren
  }
 }


